# -*- coding: utf-8 -*-
"""
django-oml tests
"""

from .models import (ModeratedModel, LogModeratedModel, STATUS_ACCEPTED,
                     STATUS_PENDING, STATUS_REJECTED)
from django.contrib.auth.models import User, Group
from django.db import models
from django.test import TestCase


class TestModel(ModeratedModel):
    test_field = models.CharField(max_length=200)


class ModeratedModelTestCase(TestCase):

    def setUp(self):
        #ipdb.set_trace()
        self.groups = [Group.objects.create(id=1, name='non_moderated')]
        self.groups.append(Group.objects.create(id=2,  name='moderated'))
        # Create a moderated user

        # Create an unmoderated user
        self.user = User.objects.create(username="moderator",
                                        password="moderator",
                                        email="example@example.com")
        self.user.group = self.groups[1]

    def _create_test_item(self, content="basic content",
                          status=STATUS_PENDING):
        item = TestModel()
        item.test_field = content
        item.status = status
        item.save()
        return item

    def test_create_moderated_content(self):
        item = self._create_test_item(content="this must be pending")
        self.assertEquals(item.status, STATUS_PENDING)
        self.assertEqual(TestModel.objects.accepted().count(), 0)
        self.assertEqual(TestModel.objects.pending().count(), 1)
        self.assertEqual(TestModel.objects.rejected().count(), 0)
        self.assertEquals(item.test_field, "this must be pending")

    def test_get_accepted_content(self):
        item = self._create_test_item(status=STATUS_ACCEPTED,
                                      content="this must be accepted")
        self.assertEqual(TestModel.objects.accepted().count(), 1)
        self.assertEqual(TestModel.objects.pending().count(), 0)
        self.assertEqual(TestModel.objects.rejected().count(), 0)
        self.assertEquals(item.test_field, "this must be accepted")

    def test_get_rejected_content(self):
        item = self._create_test_item(status=STATUS_REJECTED,
                                      content="this must be rejected")
        self.assertEqual(TestModel.objects.accepted().count(), 0)
        self.assertEqual(TestModel.objects.pending().count(), 0)
        self.assertEqual(TestModel.objects.rejected().count(), 1)
        self.assertEquals(item.test_field, "this must be rejected")

    def test_count(self):
        """
        Let's create some items with different status to figure out if count
        is working fine
        """
        self._create_test_item(status=STATUS_REJECTED, content="item rejected")
        self._create_test_item(status=STATUS_REJECTED, content="item rejected")
        self._create_test_item(status=STATUS_REJECTED, content="item rejected")
        self._create_test_item(status=STATUS_PENDING, content="item pending")
        self._create_test_item(status=STATUS_PENDING, content="item pending")
        self._create_test_item(status=STATUS_ACCEPTED, content="item accepted")
        self.assertEquals(TestModel.objects.accepted().count(), 1)
        self.assertEquals(TestModel.objects.pending().count(), 2)
        self.assertEquals(TestModel.objects.rejected().count(), 3)
        self.assertEquals(TestModel.objects.count(), 6)

    def test_model_log_moderated_content_count(self):
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item_a = self._create_test_item(status=STATUS_ACCEPTED)
        item_p = self._create_test_item(status=STATUS_PENDING)

        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item_p.accept(self.user)
        item_p.test_field = 'change_p me 1'
        item_p.save_form_log_moderated()
        item_p.define_status_of_object(self.user)

        self.assertEquals(LogModeratedModel.objects.all().count(), 1)

        item_p.test_field = 'change_p me 2'
        item_p.save_form_log_moderated()
        item_p.define_status_of_object(self.user)

        self.assertEquals(LogModeratedModel.objects.all().count(), 1)

        item_a.test_field = 'change_a me 1'
        item_a.save_form_log_moderated()
        item_a.define_status_of_object(self.user)

        self.assertEquals(LogModeratedModel.objects.all().count(), 2)

        item_a.accept(self.user)
        self.assertEquals(LogModeratedModel.objects.all().count(), 1)

        item_p.accept(self.user)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

    def test_model_method_accept_w_accepted(self):
        item = self._create_test_item(status=STATUS_ACCEPTED)
        self.assertEquals(item.status, STATUS_ACCEPTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item.accept(self.user)
        self.assertEquals(item.status, STATUS_ACCEPTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

    def test_model_method_accept_w_pending(self):
        # Check if a new pending object can be
        # accepted
        item = self._create_test_item()
        self.assertEquals(item.status, STATUS_PENDING)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item.accept(self.user)
        self.assertEquals(item.status, STATUS_ACCEPTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        # Check if an existing object can be edited
        # and accepted
        item.test_field = 'changed field'
        item.save_form_log_moderated()
        item.define_status_of_object(self.user)

        self.assertEquals(item.status, STATUS_PENDING)
        self.assertEqual(item.test_field, 'changed field')
        self.assertEquals(LogModeratedModel.objects.all().count(), 1)

        item.accept(self.user)
        self.assertEquals(item.status, STATUS_ACCEPTED)
        self.assertEqual(item.test_field, 'changed field')
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

    def test_model_method_accept_w_rejected(self):
        # Check if a new object with rejected status
        # can be accepted
        item = self._create_test_item(status=STATUS_REJECTED)
        self.assertEquals(item.status, STATUS_REJECTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item.accept(self.user)
        self.assertEquals(item.status, STATUS_ACCEPTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

    def test_model_method_reject_w_accepted(self):
        # Rejecting a rejected object should noy
        # modify it
        item = self._create_test_item(status=STATUS_ACCEPTED)
        self.assertEquals(item.status, STATUS_ACCEPTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item.reject(self.user)
        self.assertEquals(item.status, STATUS_ACCEPTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

    def test_model_method_reject_w_pending(self):
        # Check if a new pending object is deleted
        # when rejected
        item = self._create_test_item()
        self.assertEquals(item.status, STATUS_PENDING)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item.reject(self.user)
        self.assertEquals(TestModel.objects.all().count(), 0)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        # Check if an pending object with a previous
        # accepted state is reverted when rejected
        item = self._create_test_item()
        item.accept(self.user)
        self.assertEquals(item.status, STATUS_ACCEPTED)

        item.test_field = 'changed field'
        item.save_form_log_moderated()
        item.define_status_of_object(self.user)

        self.assertEquals(item.status, STATUS_PENDING)
        self.assertEqual(item.test_field, 'changed field')
        self.assertEquals(LogModeratedModel.objects.all().count(), 1)

        item.reject(self.user)
        # Refresh the item
        item_updated = TestModel.objects.get(id=item.id)
        self.assertEquals(item_updated.status, STATUS_ACCEPTED)
        self.assertEqual(item_updated.test_field, 'basic content')
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

    def test_model_method_reject_w_rejected(self):
        # Rejecting a rejected object should noy
        # modify it
        item = self._create_test_item(status=STATUS_REJECTED)
        self.assertEquals(item.status, STATUS_REJECTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)

        item.reject(self.user)
        self.assertEquals(item.status, STATUS_REJECTED)
        self.assertEquals(LogModeratedModel.objects.all().count(), 0)



from django.conf import settings
from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.core import serializers
from django.db import models
from django.utils.translation import ugettext_lazy as _
from managers import ModeratedModelManager


try:
    from django.utils import timezone
except ImportError:
    from datetime import datetime as timezone

USER_MODEL = getattr(settings, 'USER_MODEL', None) or \
             getattr(settings, 'AUTH_USER_MODEL', None) or \
             'auth.User'

STATUS_ACCEPTED = 'a'
STATUS_PENDING = 'p'
STATUS_REJECTED = 'r'
STATUS_CHOICES = (
    (STATUS_PENDING, _('Pending')),
    (STATUS_ACCEPTED, _('Accepted')),
    (STATUS_REJECTED, _('Rejected')),
)

OML_CONFIG = getattr(settings, 'OML_CONFIG', None)
OML_EXCLUDE_MODERATED = OML_CONFIG['OML_EXCLUDE_MODERATED']
OML_EXCLUDED_GROUPS = OML_CONFIG['OML_EXCLUDED_GROUPS']


class LogModeratedModel(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.IntegerField(db_index=True)
    object_dump = models.TextField()


class ModeratedModel(models.Model):
    authorized_by = models.ForeignKey(USER_MODEL, null=True, blank=True,
                                      editable=False)
    status = models.CharField(max_length=1, choices=STATUS_CHOICES,
                              default=STATUS_PENDING, editable=False,
                              db_index=True)
    status_date = models.DateTimeField(null=True, blank=True, editable=False)
    objects = ModeratedModelManager()

    def accept(self, user):
        """Set status accepted to the current item
        :param user: user who is approving the content
        """

        if self.status == STATUS_ACCEPTED:
            return

        # Search in moderated logs
        logs = LogModeratedModel.objects.filter(
            content_type=ContentType.objects.get_for_model(type(self)),
            object_id=self.id)
        # Delete the log if exists
        if logs:
            logs[0].delete()

        self.status = STATUS_ACCEPTED
        self.authorized_by = user
        self.status_date = timezone.now()
        self.save()

    def reject(self, user):
        """Set status rejected to the current item
        :param user: user who is approving the content
        """
        logs = []

        if not self.status == STATUS_PENDING:
            # Rejected=True & Deleted=False
            return (False, False)

        # Search in moderated logs
        try:
            logs = LogModeratedModel.objects.get(
                content_type=ContentType.objects.get_for_model(type(self)),
                object_id=self.id)
        except LogModeratedModel.MultipleObjectsReturned:
            return False
        except LogModeratedModel.DoesNotExist:
            pass

        if logs:
            # Replace with the last accepted
            # object and delete the log
            for obj_original in serializers.deserialize('json',
                                                        logs.object_dump):
                obj_original.save()
                logs.delete()
            # Rejected=True & Deleted=False
            return (True, False)
        # If there is no logs, then we
        # reject the creation of the object
        self.status = STATUS_REJECTED
        self.authorized_by = user
        self.status_date = timezone.now()
        self.save()
        self.delete()
        # Rejected=True & Deleted=True
        return (True, True)

    def save_form_log_moderated(self):
        if self.status == STATUS_ACCEPTED:
            # store the log of the moderated model
            content_type = ContentType.objects.get_for_model(
                self, for_concrete_model=False)
            # Save the original object in LogModeratedModel
            obj_data = serializers.serialize(
                'json', [content_type.get_object_for_this_type(id=self.id)])
            log = {
                'content_type': content_type,
                'object_id': self.id,
                'object_dump': obj_data,
            }
            LogModeratedModel.objects.create(**log)

    def define_status_of_object(self, user):
        try:
            if not OML_EXCLUDE_MODERATED or (user.group.id not in
                                             OML_EXCLUDED_GROUPS):
                self.status = STATUS_PENDING
        except AttributeError:
            # If either OML_EXCLUDE... or user
            # raises and error, there is a improper
            # configuration
            pass

    class Meta:
        abstract = True


class ModelAdminOml(admin.ModelAdmin):
    """
    Extension of ModelAdmin
    """

    def save_form(self, request, form, change, **kwargs):
        """
        Given a ModelForm return an unsaved instance. ``change`` is True if
        the object is being changed, and False if it's being added.
        """

        # Save the original object in LogModeratedModel
        #LogModeratedModel.ob

        # Store the object on the DB
        form = super(ModelAdminOml, self).save_form(request, form, change)

        status = getattr(form, 'status', None)
        # Store the log of the moderated model
        form.save_form_log_moderated()
        # Change status if necesary
        form.define_status_of_object(request.user)

        return form



from django.db import models


class ModeratedModelQuerySet(models.query.QuerySet):
    def main_queryset(self, status):
        return self.filter(status=status)


class ModeratedModelManager(models.Manager):
    def get_query_set(self):
        return ModeratedModelQuerySet(self.model, using=self._db)

    def accepted(self):
        return self.get_query_set().main_queryset(status='a')

    def rejected(self):
        return self.get_query_set().main_queryset(status='r')

    def pending(self):
        return self.get_query_set().main_queryset(status='p')
